@@ -1,11 +1,14 @@
+//! Support for querying era validators.
 use thiserror::Error;
 
 use datasize::DataSize;
 
+use casper_hashing::Digest;
 use casper_types::ProtocolVersion;
 
-use crate::{core::engine_state::error::Error, shared::newtypes::Blake2bHash};
+use crate::core::engine_state::error::Error;
 
+/// An enum that represents all possible error conditions of a `get_era_validators` request.
 #[derive(Debug, Error, DataSize)]
 pub enum GetEraValidatorsError {
     /// Invalid state hash was used to make this request
@@ -20,29 +23,34 @@ pub enum GetEraValidatorsError {
 }
 
 impl GetEraValidatorsError {
+    /// Returns `true` if the result represents missing era validators.
     pub fn is_era_validators_missing(&self) -> bool {
         matches!(self, GetEraValidatorsError::EraValidatorsMissing)
     }
 }
 
+/// Represents a `get_era_validators` request.
 #[derive(Debug, Clone, PartialEq, Eq)]
 pub struct GetEraValidatorsRequest {
-    state_hash: Blake2bHash,
+    state_hash: Digest,
     protocol_version: ProtocolVersion,
 }
 
 impl GetEraValidatorsRequest {
-    pub fn new(state_hash: Blake2bHash, protocol_version: ProtocolVersion) -> Self {
+    /// Creates a new [`GetEraValidatorsRequest`].
+    pub fn new(state_hash: Digest, protocol_version: ProtocolVersion) -> Self {
         GetEraValidatorsRequest {
             state_hash,
             protocol_version,
         }
     }
 
-    pub fn state_hash(&self) -> Blake2bHash {
+    /// Returns the state root hash.
+    pub fn state_hash(&self) -> Digest {
         self.state_hash
     }
 
+    /// Returns the protocol version.
     pub fn protocol_version(&self) -> ProtocolVersion {
         self.protocol_version
     }
