@@ -1,13 +1,14 @@
+//! Outcome of an `ExecutionRequest`.
 use std::collections::VecDeque;
 
-use casper_types::{bytesrepr::FromBytes, CLTyped, CLValue, CLValueError, Key, TransferAddr};
+use casper_types::{
+    bytesrepr::FromBytes, CLTyped, CLValue, Gas, Key, Motes, StoredValue, TransferAddr,
+};
 
 use super::{error, execution_effect::ExecutionEffect, op::Op};
 use crate::{
-    shared::{
-        additive_map::AdditiveMap, gas::Gas, motes::Motes, newtypes::CorrelationId,
-        stored_value::StoredValue, transform::Transform,
-    },
+    core::execution::Error as ExecError,
+    shared::{additive_map::AdditiveMap, newtypes::CorrelationId, transform::Transform},
     storage::global_state::StateReader,
 };
 
@@ -16,13 +17,15 @@ fn make_payment_error_effects(
     account_main_purse_balance: Motes,
     account_main_purse_balance_key: Key,
     proposer_main_purse_balance_key: Key,
-) -> Result<ExecutionEffect, CLValueError> {
+) -> Result<ExecutionEffect, error::Error> {
     let mut ops = AdditiveMap::new();
     let mut transforms = AdditiveMap::new();
 
-    let new_balance = account_main_purse_balance - max_payment_cost;
+    let new_balance = account_main_purse_balance
+        .checked_sub(max_payment_cost)
+        .ok_or(error::Error::InsufficientPayment)?;
     // from_t for U512 is assumed to never panic
-    let new_balance_clvalue = CLValue::from_t(new_balance.value())?;
+    let new_balance_clvalue = CLValue::from_t(new_balance.value()).map_err(ExecError::from)?;
     let new_balance_value = StoredValue::CLValue(new_balance_clvalue);
 
     let account_main_purse_balance_normalize = account_main_purse_balance_key.normalize();
@@ -43,19 +46,28 @@ fn make_payment_error_effects(
     Ok(ExecutionEffect::new(ops, transforms))
 }
 
+/// Represents the result of an execution specified by
+/// [`crate::core::engine_state::ExecuteRequest`].
 #[derive(Clone, Debug)]
 pub enum ExecutionResult {
     /// An error condition that happened during execution
     Failure {
+        /// Error causing this `Failure` variant.
         error: error::Error,
-        effect: ExecutionEffect,
+        /// Contains all the effects that occurred during execution up to the point of the failure.
+        execution_effect: ExecutionEffect,
+        /// List of transfers that happened during execution up to the point of the failure.
         transfers: Vec<TransferAddr>,
+        /// Gas consumed up to the point of the failure.
         cost: Gas,
     },
     /// Execution was finished successfully
     Success {
-        effect: ExecutionEffect,
+        /// Execution effect.
+        execution_effect: ExecutionEffect,
+        /// List of transfers.
         transfers: Vec<TransferAddr>,
+        /// Gas cost.
         cost: Gas,
     },
 }
@@ -63,7 +75,7 @@ pub enum ExecutionResult {
 impl Default for ExecutionResult {
     fn default() -> Self {
         ExecutionResult::Success {
-            effect: ExecutionEffect::default(),
+            execution_effect: ExecutionEffect::default(),
             transfers: Vec::default(),
             cost: Gas::default(),
         }
@@ -73,6 +85,7 @@ impl Default for ExecutionResult {
 /// A type alias that represents multiple execution results.
 pub type ExecutionResults = VecDeque<ExecutionResult>;
 
+/// Indicates the outcome of a transfer payment check.
 pub enum ForcedTransferResult {
     /// Payment code ran out of gas during execution
     InsufficientPayment,
@@ -89,12 +102,13 @@ impl ExecutionResult {
     pub fn precondition_failure(error: error::Error) -> ExecutionResult {
         ExecutionResult::Failure {
             error,
-            effect: Default::default(),
+            execution_effect: Default::default(),
             transfers: Vec::default(),
             cost: Gas::default(),
         }
     }
 
+    /// Returns `true` if this is a successful variant.
     pub fn is_success(&self) -> bool {
         match self {
             ExecutionResult::Failure { .. } => false,
@@ -102,6 +116,7 @@ impl ExecutionResult {
         }
     }
 
+    /// Returns `true` if this is a failure variant.
     pub fn is_failure(&self) -> bool {
         match self {
             ExecutionResult::Failure { .. } => true,
@@ -109,15 +124,22 @@ impl ExecutionResult {
         }
     }
 
+    /// Returns `true` if this is a precondition failure.
+    ///
+    /// Precondition variant is further described as an execution failure which does not have any
+    /// effects, and has a gas cost of 0.
     pub fn has_precondition_failure(&self) -> bool {
         match self {
-            ExecutionResult::Failure { cost, effect, .. } => {
-                cost.value() == 0.into() && *effect == Default::default()
-            }
+            ExecutionResult::Failure {
+                cost,
+                execution_effect,
+                ..
+            } => cost.value() == 0.into() && *execution_effect == Default::default(),
             ExecutionResult::Success { .. } => false,
         }
     }
 
+    /// Returns gas cost of execution regardless of variant.
     pub fn cost(&self) -> Gas {
         match self {
             ExecutionResult::Failure { cost, .. } => *cost,
@@ -125,13 +147,19 @@ impl ExecutionResult {
         }
     }
 
+    /// Returns execution effects regardless of variant.
     pub fn effect(&self) -> &ExecutionEffect {
         match self {
-            ExecutionResult::Failure { effect, .. } => effect,
-            ExecutionResult::Success { effect, .. } => effect,
+            ExecutionResult::Failure {
+                execution_effect, ..
+            } => execution_effect,
+            ExecutionResult::Success {
+                execution_effect, ..
+            } => execution_effect,
         }
     }
 
+    /// Returns list of transfers regardless of variant.
     pub fn transfers(&self) -> &Vec<TransferAddr> {
         match self {
             ExecutionResult::Failure { transfers, .. } => transfers,
@@ -139,30 +167,40 @@ impl ExecutionResult {
         }
     }
 
+    /// Returns a new execution result with updated gas cost.
+    ///
+    /// This method supports preserves the [`ExecutionResult`] variant and updates the cost field
+    /// only.
     pub fn with_cost(self, cost: Gas) -> Self {
         match self {
             ExecutionResult::Failure {
                 error,
-                effect,
+                execution_effect,
                 transfers,
                 ..
             } => ExecutionResult::Failure {
                 error,
-                effect,
+                execution_effect,
                 transfers,
                 cost,
             },
             ExecutionResult::Success {
-                effect, transfers, ..
+                execution_effect,
+                transfers,
+                ..
             } => ExecutionResult::Success {
-                effect,
+                execution_effect,
                 transfers,
                 cost,
             },
         }
     }
 
-    pub fn with_effect(self, effect: ExecutionEffect) -> Self {
+    /// Returns a new execution result with updated execution effects.
+    ///
+    /// This method preserves the [`ExecutionResult`] variant and updates the
+    /// `execution_effect` field only.
+    pub fn with_effect(self, execution_effect: ExecutionEffect) -> Self {
         match self {
             ExecutionResult::Failure {
                 error,
@@ -171,41 +209,53 @@ impl ExecutionResult {
                 ..
             } => ExecutionResult::Failure {
                 error,
-                effect,
+                execution_effect,
                 transfers,
                 cost,
             },
             ExecutionResult::Success {
                 cost, transfers, ..
             } => ExecutionResult::Success {
-                effect,
+                execution_effect,
                 transfers,
                 cost,
             },
         }
     }
 
+    /// Returns a new execution result with updated transfers field.
+    ///
+    /// This method supports preserves the [`ExecutionResult`] variant and updates the
+    /// `transfers` field only.
     pub fn with_transfers(self, transfers: Vec<TransferAddr>) -> Self {
         match self {
             ExecutionResult::Failure {
                 error,
-                effect,
+                execution_effect,
                 cost,
                 ..
             } => ExecutionResult::Failure {
                 error,
-                effect,
+                execution_effect,
                 transfers,
                 cost,
             },
-            ExecutionResult::Success { cost, effect, .. } => ExecutionResult::Success {
-                effect,
+            ExecutionResult::Success {
+                cost,
+                execution_effect,
+                ..
+            } => ExecutionResult::Success {
+                execution_effect,
                 transfers,
                 cost,
             },
         }
     }
 
+    /// Returns error value, if possible.
+    ///
+    /// Returns a reference to a wrapped [`error::Error`] instance if the object is a failure
+    /// variant.
     pub fn as_error(&self) -> Option<&error::Error> {
         match self {
             ExecutionResult::Failure { error, .. } => Some(error),
@@ -222,6 +272,15 @@ impl ExecutionResult {
         }
     }
 
+    /// Checks the transfer status of a payment code.
+    ///
+    /// This method converts the gas cost of the execution result into motes using supplied
+    /// `gas_price`, and then a check is made to ensure that user deposited enough funds in the
+    /// payment purse (in motes) to cover the execution of a payment code.
+    ///
+    /// Returns `None` if user deposited enough funds in payment purse and the execution result was
+    /// a success variant, otherwise a wrapped [`ForcedTransferResult`] that indicates an error
+    /// codition.
     pub fn check_forced_transfer(
         &self,
         payment_purse_balance: Motes,
@@ -251,6 +310,13 @@ impl ExecutionResult {
         }
     }
 
+    /// Creates a new payment code error.
+    ///
+    /// The method below creates an [`ExecutionResult`] with precomputed effects of a
+    /// "finalize_payment".
+    ///
+    /// The effects that are produced as part of this process would subract `max_payment_cost` from
+    /// account's main purse, and add `max_payment_cost` to proposer account's balance.
     pub fn new_payment_code_error(
         error: error::Error,
         max_payment_cost: Motes,
@@ -258,8 +324,8 @@ impl ExecutionResult {
         gas_cost: Gas,
         account_main_purse_balance_key: Key,
         proposer_main_purse_balance_key: Key,
-    ) -> Result<ExecutionResult, CLValueError> {
-        let effect = make_payment_error_effects(
+    ) -> Result<ExecutionResult, error::Error> {
+        let execution_effect = make_payment_error_effects(
             max_payment_cost,
             account_main_purse_balance,
             account_main_purse_balance_key,
@@ -268,17 +334,19 @@ impl ExecutionResult {
         let transfers = Vec::default();
         Ok(ExecutionResult::Failure {
             error,
-            effect,
+            execution_effect,
             transfers,
             cost: gas_cost,
         })
     }
 
-    pub fn take_with_ret<T: FromBytes + CLTyped>(self, ret: T) -> (Option<T>, Self) {
+    /// Returns a wrapped `ret` by consuming object.
+    pub(crate) fn take_with_ret<T: FromBytes + CLTyped>(self, ret: T) -> (Option<T>, Self) {
         (Some(ret), self)
     }
 
-    pub fn take_without_ret<T: FromBytes + CLTyped>(self) -> (Option<T>, Self) {
+    /// Returns a self and has a return type compatible with [`ExecutionResult::take_with_ret`].
+    pub(crate) fn take_without_ret<T: FromBytes + CLTyped>(self) -> (Option<T>, Self) {
         (None, self)
     }
 }
@@ -287,21 +355,21 @@ impl From<&ExecutionResult> for casper_types::ExecutionResult {
     fn from(ee_execution_result: &ExecutionResult) -> Self {
         match ee_execution_result {
             ExecutionResult::Success {
-                effect,
+                execution_effect,
                 transfers,
                 cost,
             } => casper_types::ExecutionResult::Success {
-                effect: effect.into(),
+                effect: execution_effect.into(),
                 transfers: transfers.clone(),
                 cost: cost.value(),
             },
             ExecutionResult::Failure {
                 error,
-                effect,
+                execution_effect,
                 transfers,
                 cost,
             } => casper_types::ExecutionResult::Failure {
-                effect: effect.into(),
+                effect: execution_effect.into(),
                 transfers: transfers.clone(),
                 cost: cost.value(),
                 error_message: error.to_string(),
@@ -310,13 +378,19 @@ impl From<&ExecutionResult> for casper_types::ExecutionResult {
     }
 }
 
+/// Represents error conditions of an execution result builder.
 #[derive(Debug, Copy, Clone, PartialEq, Eq)]
 pub enum ExecutionResultBuilderError {
+    /// Missing a payment execution result.
     MissingPaymentExecutionResult,
+    /// Missing a session execution result.
     MissingSessionExecutionResult,
+    /// Missing a finalize execution result.
     MissingFinalizeExecutionResult,
 }
 
+/// Builder object that will construct a final [`ExecutionResult`] given payment, session and
+/// finalize [`ExecutionResult`]s.
 pub struct ExecutionResultBuilder {
     payment_execution_result: Option<ExecutionResult>,
     session_execution_result: Option<ExecutionResult>,
@@ -334,15 +408,18 @@ impl Default for ExecutionResultBuilder {
 }
 
 impl ExecutionResultBuilder {
+    /// Creates new execution result builder.
     pub fn new() -> ExecutionResultBuilder {
         ExecutionResultBuilder::default()
     }
 
+    /// Sets a payment execution result.
     pub fn set_payment_execution_result(&mut self, payment_result: ExecutionResult) -> &mut Self {
         self.payment_execution_result = Some(payment_result);
         self
     }
 
+    /// Sets a session execution result.
     pub fn set_session_execution_result(
         &mut self,
         session_execution_result: ExecutionResult,
@@ -351,6 +428,7 @@ impl ExecutionResultBuilder {
         self
     }
 
+    /// Sets a finalize execution result.
     pub fn set_finalize_execution_result(
         &mut self,
         finalize_execution_result: ExecutionResult,
@@ -359,6 +437,10 @@ impl ExecutionResultBuilder {
         self
     }
 
+    /// Calculates the total gas cost of the execution result.
+    ///
+    /// Takes a payment execution result, and a session execution result and returns a sum. If
+    /// either a payment or session code is not specified then a 0 is used.
     pub fn total_cost(&self) -> Gas {
         let payment_cost = self
             .payment_execution_result
@@ -370,9 +452,16 @@ impl ExecutionResultBuilder {
             .as_ref()
             .map(ExecutionResult::cost)
             .unwrap_or_default();
+        // TODO: Make sure this code isn't in production, as, even though it's highly unlikely
+        // to happen, an integer overflow would be silently ignored in release builds.
+        // NOTE: This code should have been removed in the fix of #1968, where arithmetic
+        // operations on the Gas type were disabled.
         payment_cost + session_cost
     }
 
+    /// Returns transfers from a session's execution result.
+    ///
+    /// If the session's execution result is not supplied then an empty [`Vec`] is returned.
     pub fn transfers(&self) -> Vec<TransferAddr> {
         self.session_execution_result
             .as_ref()
@@ -381,6 +470,8 @@ impl ExecutionResultBuilder {
             .unwrap_or_default()
     }
 
+    /// Builds a final [`ExecutionResult`] based on session result, payment result and a
+    /// finalization result.
     pub fn build<R: StateReader<Key, StoredValue>>(
         self,
         reader: &R,
@@ -392,7 +483,7 @@ impl ExecutionResultBuilder {
         let mut transforms = AdditiveMap::new();
 
         let mut ret: ExecutionResult = ExecutionResult::Success {
-            effect: Default::default(),
+            execution_effect: Default::default(),
             transfers,
             cost,
         };
