@@ -1,17 +1,20 @@
+//! Support for a genesis process.
 use std::{cell::RefCell, collections::BTreeMap, fmt, iter, rc::Rc};
 
 use datasize::DataSize;
+use num::Zero;
 use num_rational::Ratio;
-use num_traits::Zero;
 use parity_wasm::elements::Module;
 use rand::{
     distributions::{Distribution, Standard},
     Rng,
 };
 use serde::{Deserialize, Serialize};
+use tracing::error;
 
+use casper_hashing::Digest;
 use casper_types::{
-    account::AccountHash,
+    account::{Account, AccountHash},
     bytesrepr::{self, FromBytes, ToBytes, U8_SERIALIZED_LENGTH},
     contracts::{ContractPackageStatus, ContractVersions, DisabledVersions, Groups, NamedKeys},
     runtime_args,
@@ -23,16 +26,16 @@ use casper_types::{
             INITIAL_ERA_END_TIMESTAMP_MILLIS, INITIAL_ERA_ID, LOCKED_FUNDS_PERIOD_KEY,
             SEIGNIORAGE_RECIPIENTS_SNAPSHOT_KEY, UNBONDING_DELAY_KEY, VALIDATOR_SLOTS_KEY,
         },
-        handle_payment::{self},
+        handle_payment,
         mint::{
             self, ARG_AMOUNT, ARG_ROUND_SEIGNIORAGE_RATE, METHOD_MINT, ROUND_SEIGNIORAGE_RATE_KEY,
             TOTAL_SUPPLY_KEY,
         },
-        standard_payment,
+        standard_payment, AUCTION, HANDLE_PAYMENT, MINT, STANDARD_PAYMENT,
     },
     AccessRights, CLValue, Contract, ContractHash, ContractPackage, ContractPackageHash,
-    ContractWasm, ContractWasmHash, DeployHash, EntryPointType, EntryPoints, EraId, Key, Phase,
-    ProtocolVersion, PublicKey, RuntimeArgs, SecretKey, URef, U512,
+    ContractWasm, ContractWasmHash, DeployHash, EntryPointType, EntryPoints, EraId, Gas, Key,
+    Motes, Phase, ProtocolVersion, PublicKey, RuntimeArgs, SecretKey, StoredValue, URef, U512,
 };
 
 use crate::{
@@ -40,69 +43,33 @@ use crate::{
         engine_state::{execution_effect::ExecutionEffect, EngineConfig},
         execution,
         execution::{AddressGenerator, Executor},
-        tracking_copy::TrackingCopy,
-    },
-    shared::{
-        account::Account,
-        gas::Gas,
-        motes::Motes,
-        newtypes::{Blake2bHash, CorrelationId},
-        stored_value::StoredValue,
-        system_config::SystemConfig,
-        wasm_config::WasmConfig,
-        TypeMismatch,
-    },
-    storage::{
-        global_state::{CommitResult, StateProvider},
-        protocol_data::ProtocolData,
+        tracking_copy::{TrackingCopy, TrackingCopyExt},
     },
+    shared::{newtypes::CorrelationId, system_config::SystemConfig, wasm_config::WasmConfig},
+    storage::global_state::StateProvider,
 };
 
-pub const PLACEHOLDER_KEY: Key = Key::Hash([0u8; 32]);
 const TAG_LENGTH: usize = U8_SERIALIZED_LENGTH;
+const DEFAULT_ADDRESS: [u8; 32] = [0; 32];
+/// Type alias for the system contract registry.
+pub type SystemContractRegistry = BTreeMap<String, ContractHash>;
 
-#[derive(Debug, Serialize)]
-pub enum GenesisResult {
-    RootNotFound,
-    KeyNotFound(Key),
-    TypeMismatch(TypeMismatch),
-    Serialization(bytesrepr::Error),
-    Success {
-        post_state_hash: Blake2bHash,
-        #[serde(skip_serializing)]
-        effect: ExecutionEffect,
-    },
+/// Represents an outcome of a successful genesis run.
+#[derive(Debug)]
+pub struct GenesisSuccess {
+    /// State hash after genesis is committed to the global state.
+    pub post_state_hash: Digest,
+    /// Effects of a successful genesis.
+    pub execution_effect: ExecutionEffect,
 }
 
-impl fmt::Display for GenesisResult {
+impl fmt::Display for GenesisSuccess {
     fn fmt(&self, f: &mut fmt::Formatter) -> Result<(), fmt::Error> {
-        match self {
-            GenesisResult::RootNotFound => write!(f, "Root not found"),
-            GenesisResult::KeyNotFound(key) => write!(f, "Key not found: {}", key),
-            GenesisResult::TypeMismatch(type_mismatch) => {
-                write!(f, "Type mismatch: {:?}", type_mismatch)
-            }
-            GenesisResult::Serialization(error) => write!(f, "Serialization error: {:?}", error),
-            GenesisResult::Success {
-                post_state_hash,
-                effect,
-            } => write!(f, "Success: {} {:?}", post_state_hash, effect),
-        }
-    }
-}
-
-impl GenesisResult {
-    pub fn from_commit_result(commit_result: CommitResult, effect: ExecutionEffect) -> Self {
-        match commit_result {
-            CommitResult::RootNotFound => GenesisResult::RootNotFound,
-            CommitResult::KeyNotFound(key) => GenesisResult::KeyNotFound(key),
-            CommitResult::TypeMismatch(type_mismatch) => GenesisResult::TypeMismatch(type_mismatch),
-            CommitResult::Serialization(error) => GenesisResult::Serialization(error),
-            CommitResult::Success { state_root, .. } => GenesisResult::Success {
-                post_state_hash: state_root,
-                effect,
-            },
-        }
+        write!(
+            f,
+            "Success: {} {:?}",
+            self.post_state_hash, self.execution_effect
+        )
     }
 }
 
@@ -113,9 +80,12 @@ enum GenesisAccountTag {
     Delegator = 2,
 }
 
+/// Represents details about genesis account's validator status.
 #[derive(DataSize, Debug, Copy, Clone, PartialEq, Eq, Serialize, Deserialize)]
 pub struct GenesisValidator {
+    /// Stake of a genesis validator.
     bonded_amount: Motes,
+    /// Delegation rate in the range of 0-100.
     delegation_rate: DelegationRate,
 }
 
@@ -145,6 +115,7 @@ impl FromBytes for GenesisValidator {
 }
 
 impl GenesisValidator {
+    /// Creates new [`GenesisValidator`].
     pub fn new(bonded_amount: Motes, delegation_rate: DelegationRate) -> Self {
         Self {
             bonded_amount,
@@ -152,10 +123,12 @@ impl GenesisValidator {
         }
     }
 
+    /// Returns the bonded amount of a genesis validator.
     pub fn bonded_amount(&self) -> Motes {
         self.bonded_amount
     }
 
+    /// Returns the delegation rate of a genesis validator.
     pub fn delegation_rate(&self) -> DelegationRate {
         self.delegation_rate
     }
@@ -170,18 +143,33 @@ impl Distribution<GenesisValidator> for Standard {
     }
 }
 
+/// This enum represents possible states of a genesis account.
 #[derive(DataSize, Debug, Clone, PartialEq, Eq, Serialize, Deserialize)]
 pub enum GenesisAccount {
+    /// This variant is for internal use only - genesis process will create a virtual system
+    /// account and use it to call system contracts.
     System,
+    /// Genesis account that will be created.
     Account {
+        /// Public key of a genesis account.
         public_key: PublicKey,
+        /// Starting balance of a genesis account.
         balance: Motes,
+        /// If set, it will make this account a genesis validator.
         validator: Option<GenesisValidator>,
     },
+    /// The genesis delegator is a special account that will be created as a delegator.
+    /// It does not have any stake of its own, but will create a real account in the system
+    /// which will delegate to a genesis validator.
     Delegator {
+        /// Validator's public key that has to refer to other instance of
+        /// [`GenesisAccount::Account`] with a `validator` field set.
         validator_public_key: PublicKey,
+        /// Public key of the genesis account that will be created as part of this entry.
         delegator_public_key: PublicKey,
+        /// Starting balance of the account.
         balance: Motes,
+        /// Delegated amount for given `validator_public_key`.
         delegated_amount: Motes,
     },
 }
@@ -454,6 +442,7 @@ impl FromBytes for GenesisAccount {
     }
 }
 
+/// Represents a configuration of a genesis process.
 #[derive(Debug, Clone, PartialEq, Eq)]
 pub struct GenesisConfig {
     name: String,
@@ -463,6 +452,7 @@ pub struct GenesisConfig {
 }
 
 impl GenesisConfig {
+    /// Creates a new genesis config object.
     pub fn new(
         name: String,
         timestamp: u64,
@@ -477,26 +467,32 @@ impl GenesisConfig {
         }
     }
 
+    /// Returns name.
     pub fn name(&self) -> &str {
         self.name.as_str()
     }
 
+    /// Returns timestamp.
     pub fn timestamp(&self) -> u64 {
         self.timestamp
     }
 
+    /// Returns protocol version.
     pub fn protocol_version(&self) -> ProtocolVersion {
         self.protocol_version
     }
 
+    /// Returns configuration details of the genesis process.
     pub fn ee_config(&self) -> &ExecConfig {
         &self.ee_config
     }
 
+    /// Returns mutable reference to the configuration.
     pub fn ee_config_mut(&mut self) -> &mut ExecConfig {
         &mut self.ee_config
     }
 
+    /// Returns genesis configuration and consumes the object.
     pub fn take_ee_config(self) -> ExecConfig {
         self.ee_config
     }
@@ -525,6 +521,7 @@ impl Distribution<GenesisConfig> for Standard {
     }
 }
 
+/// Represents the details of a genesis process.
 #[derive(Clone, Debug, PartialEq, Eq, Serialize, Deserialize)]
 pub struct ExecConfig {
     accounts: Vec<GenesisAccount>,
@@ -539,6 +536,7 @@ pub struct ExecConfig {
 }
 
 impl ExecConfig {
+    /// Creates new genesis configuration.
     #[allow(clippy::too_many_arguments)]
     pub fn new(
         accounts: Vec<GenesisAccount>,
@@ -564,20 +562,24 @@ impl ExecConfig {
         }
     }
 
+    /// Returns WASM config.
     pub fn wasm_config(&self) -> &WasmConfig {
         &self.wasm_config
     }
 
+    /// Returns system config.
     pub fn system_config(&self) -> &SystemConfig {
         &self.system_config
     }
 
+    /// Returns all bonded genesis validators.
     pub fn get_bonded_validators(&self) -> impl Iterator<Item = &GenesisAccount> {
         self.accounts
             .iter()
             .filter(|&genesis_account| genesis_account.is_validator())
     }
 
+    /// Returns all bonded genesis delegators.
     pub fn get_bonded_delegators(
         &self,
     ) -> impl Iterator<Item = (&PublicKey, &PublicKey, &Motes, &Motes)> {
@@ -586,34 +588,42 @@ impl ExecConfig {
             .filter_map(|genesis_account| genesis_account.as_delegator())
     }
 
+    /// Returns all genesis accounts.
     pub fn accounts(&self) -> &[GenesisAccount] {
         self.accounts.as_slice()
     }
 
+    /// Adds new genesis account to the config.
     pub fn push_account(&mut self, account: GenesisAccount) {
         self.accounts.push(account)
     }
 
+    /// Returns validator slots.
     pub fn validator_slots(&self) -> u32 {
         self.validator_slots
     }
 
+    /// Returns auction delay.
     pub fn auction_delay(&self) -> u64 {
         self.auction_delay
     }
 
+    /// Returns locked funds period expressed in milliseconds.
     pub fn locked_funds_period_millis(&self) -> u64 {
         self.locked_funds_period_millis
     }
 
+    /// Returns round seigniorage rate.
     pub fn round_seigniorage_rate(&self) -> Ratio<u64> {
         self.round_seigniorage_rate
     }
 
+    /// Returns unbonding delay in eras.
     pub fn unbonding_delay(&self) -> u64 {
         self.unbonding_delay
     }
 
+    /// Returns genesis timestamp expressed in milliseconds.
     pub fn genesis_timestamp_millis(&self) -> u64 {
         self.genesis_timestamp_millis
     }
@@ -658,34 +668,65 @@ impl Distribution<ExecConfig> for Standard {
     }
 }
 
+/// Error returned as a result of a failed genesis process.
 #[derive(Clone, Debug)]
 pub enum GenesisError {
+    /// Error creating a runtime.
     UnableToCreateRuntime,
+    /// Error obtaining the mint's contract key.
     InvalidMintKey,
+    /// Missing mint contract.
     MissingMintContract,
+    /// Unexpected stored value variant.
     UnexpectedStoredValue,
-    MissingPublicKey,
+    /// Error executing a system contract.
     ExecutionError(execution::Error),
+    /// Error executing the mint system contract.
     MintError(mint::Error),
+    /// Error converting a [`CLValue`] to a concrete type.
     CLValue(String),
+    /// Specified validator does not exist among the genesis accounts.
     OrphanedDelegator {
+        /// Validator's public key.
         validator_public_key: PublicKey,
+        /// Delegator's public key.
         delegator_public_key: PublicKey,
     },
+    /// Duplicated delegator entry found for a given validator.
     DuplicatedDelegatorEntry {
+        /// Validator's public key.
         validator_public_key: PublicKey,
+        /// Delegator's public key.
         delegator_public_key: PublicKey,
     },
+    /// Delegation rate outside the allowed range.
     InvalidDelegationRate {
+        /// Delegator's public key.
         public_key: PublicKey,
+        /// Invalid delegation rate specified in the genesis account entry.
         delegation_rate: DelegationRate,
     },
+    /// Invalid bond amount in a genesis account.
     InvalidBondAmount {
+        /// Validator's public key.
         public_key: PublicKey,
     },
+    /// Invalid delegated amount in a genesis account.
     InvalidDelegatedAmount {
+        /// Delegator's public key.
         public_key: PublicKey,
     },
+    /// Failed to create system registry.
+    FailedToCreateSystemRegistry,
+    /// Missing system contract hash.
+    MissingSystemContractHash(String),
+    /// Invalid number of validator slots configured.
+    InvalidValidatorSlots {
+        /// Number of validators in the genesis config.
+        validators: usize,
+        /// Number of validator slots specified.
+        validator_slots: u32,
+    },
 }
 
 pub(crate) struct GenesisInstaller<S>
@@ -693,7 +734,7 @@ where
     S: StateProvider,
     S::Error: Into<execution::Error>,
 {
-    genesis_config_hash: Blake2bHash,
+    genesis_config_hash: Digest,
     virtual_system_account: Account,
     protocol_version: ProtocolVersion,
     correlation_id: CorrelationId,
@@ -703,7 +744,6 @@ where
     transfer_address_generator: Rc<RefCell<AddressGenerator>>,
     executor: Executor,
     tracking_copy: Rc<RefCell<TrackingCopy<<S as StateProvider>::Reader>>>,
-    protocol_data: ProtocolData,
     system_module: Module,
 }
 
@@ -713,7 +753,7 @@ where
     S::Error: Into<execution::Error>,
 {
     pub(crate) fn new(
-        genesis_config_hash: Blake2bHash,
+        genesis_config_hash: Digest,
         protocol_version: ProtocolVersion,
         correlation_id: CorrelationId,
         engine_config: EngineConfig,
@@ -738,8 +778,6 @@ where
             Rc::new(RefCell::new(generator))
         };
 
-        let protocol_data = ProtocolData::default();
-
         let system_account_addr = PublicKey::System.to_account_hash();
 
         let virtual_system_account = {
@@ -764,7 +802,6 @@ where
             transfer_address_generator,
             executor,
             tracking_copy,
-            protocol_data,
             system_module,
         }
     }
@@ -832,7 +869,20 @@ where
 
         let (_, mint_hash) = self.store_contract(access_key, named_keys, entry_points);
 
-        self.protocol_data = ProtocolData::partial_with_mint(mint_hash);
+        {
+            // Insert a partial registry into global state.
+            // This allows for default values to be accessible when the remaining system contracts
+            // call the `call_host_mint` function during their creation.
+            let mut partial_registry = BTreeMap::<String, ContractHash>::new();
+            partial_registry.insert(MINT.to_string(), mint_hash);
+            partial_registry.insert(HANDLE_PAYMENT.to_string(), DEFAULT_ADDRESS.into());
+            let cl_registry = CLValue::from_t(partial_registry)
+                .map_err(|error| GenesisError::CLValue(error.to_string()))?;
+            self.tracking_copy.borrow_mut().write(
+                Key::SystemContractRegistry,
+                StoredValue::CLValue(cl_registry),
+            );
+        }
 
         Ok(mint_hash)
     }
@@ -859,6 +909,8 @@ where
 
         let (_, handle_payment_hash) = self.store_contract(access_key, named_keys, entry_points);
 
+        self.store_system_contract(HANDLE_PAYMENT, handle_payment_hash)?;
+
         Ok(handle_payment_hash)
     }
 
@@ -870,6 +922,12 @@ where
         let mut named_keys = NamedKeys::new();
 
         let genesis_validators: Vec<_> = self.exec_config.get_bonded_validators().collect();
+        if (self.exec_config.validator_slots() as usize) < genesis_validators.len() {
+            return Err(GenesisError::InvalidValidatorSlots {
+                validators: genesis_validators.len(),
+                validator_slots: self.exec_config.validator_slots(),
+            });
+        }
 
         let genesis_delegators: Vec<_> = self.exec_config.get_bonded_delegators().collect();
 
@@ -1094,10 +1152,12 @@ where
 
         let (_, auction_hash) = self.store_contract(access_key, named_keys, entry_points);
 
+        self.store_system_contract(AUCTION, auction_hash)?;
+
         Ok(auction_hash)
     }
 
-    pub(crate) fn create_standard_payment(&self) -> ContractHash {
+    pub(crate) fn create_standard_payment(&self) -> Result<ContractHash, GenesisError> {
         let named_keys = NamedKeys::new();
 
         let entry_points = standard_payment::standard_payment_entry_points();
@@ -1109,7 +1169,9 @@ where
 
         let (_, standard_payment_hash) = self.store_contract(access_key, named_keys, entry_points);
 
-        standard_payment_hash
+        self.store_system_contract(STANDARD_PAYMENT, standard_payment_hash)?;
+
+        Ok(standard_payment_hash)
     }
 
     pub(crate) fn create_accounts(&self) -> Result<(), GenesisError> {
@@ -1168,7 +1230,19 @@ where
             ARG_AMOUNT => amount,
         };
 
-        let base_key = Key::Hash(self.protocol_data.mint().value());
+        let registry = self
+            .tracking_copy
+            .borrow_mut()
+            .get_system_contracts(self.correlation_id)
+            .map_err(execution::Error::from)
+            .map_err(GenesisError::ExecutionError)?;
+
+        let mint_hash = registry.get(MINT).ok_or_else(|| {
+            error!("Missing system mint contract hash");
+            GenesisError::MissingSystemContractHash(MINT.to_string())
+        })?;
+
+        let base_key = Key::Hash(mint_hash.value());
         let mint = {
             if let StoredValue::Contract(contract) = self
                 .tracking_copy
@@ -1208,14 +1282,13 @@ where
                 self.correlation_id,
                 Rc::clone(&self.tracking_copy),
                 Phase::System,
-                self.protocol_data,
                 Default::default(),
                 call_stack,
             )
             .map_err(|_| GenesisError::UnableToCreateRuntime)?;
 
         let purse_uref = runtime
-            .call_contract(self.protocol_data.mint(), METHOD_MINT, args)
+            .call_contract(*mint_hash, METHOD_MINT, args)
             .map_err(GenesisError::ExecutionError)?
             .into_t::<Result<URef, mint::Error>>()
             .map_err(|cl_value_error| GenesisError::CLValue(cl_value_error.to_string()))?
@@ -1273,6 +1346,34 @@ where
 
         (contract_package_hash, contract_hash)
     }
+
+    fn store_system_contract(
+        &self,
+        contract_name: &str,
+        contract_hash: ContractHash,
+    ) -> Result<(), GenesisError> {
+        let partial_cl_registry = self
+            .tracking_copy
+            .borrow_mut()
+            .read(self.correlation_id, &Key::SystemContractRegistry)
+            .map_err(|_| GenesisError::FailedToCreateSystemRegistry)?
+            .ok_or_else(|| {
+                GenesisError::CLValue("failed to convert registry as stored value".to_string())
+            })?
+            .as_cl_value()
+            .ok_or_else(|| GenesisError::CLValue("failed to convert to CLValue".to_string()))?
+            .to_owned();
+        let mut partial_registry = CLValue::into_t::<SystemContractRegistry>(partial_cl_registry)
+            .map_err(|error| GenesisError::CLValue(error.to_string()))?;
+        partial_registry.insert(contract_name.to_string(), contract_hash);
+        let cl_registry = CLValue::from_t(partial_registry)
+            .map_err(|error| GenesisError::CLValue(error.to_string()))?;
+        self.tracking_copy.borrow_mut().write(
+            Key::SystemContractRegistry,
+            StoredValue::CLValue(cl_registry),
+        );
+        Ok(())
+    }
 }
 
 #[cfg(test)]
