@@ -1,87 +1,50 @@
+//! Support for applying upgrades on the execution engine.
 use std::{cell::RefCell, collections::BTreeMap, fmt, rc::Rc};
 
 use num_rational::Ratio;
 use thiserror::Error;
 
+use casper_hashing::Digest;
 use casper_types::{
     bytesrepr,
     system::{
         auction, handle_payment, mint, standard_payment, AUCTION, HANDLE_PAYMENT, MINT,
         STANDARD_PAYMENT,
     },
-    Contract, ContractHash, EntryPoints, EraId, Key, ProtocolVersion,
+    Contract, ContractHash, EntryPoints, EraId, Key, ProtocolVersion, StoredValue,
 };
 
 use crate::{
     core::{engine_state::execution_effect::ExecutionEffect, tracking_copy::TrackingCopy},
-    shared::{
-        newtypes::{Blake2bHash, CorrelationId},
-        stored_value::StoredValue,
-        system_config::SystemConfig,
-        wasm_config::WasmConfig,
-        TypeMismatch,
-    },
-    storage::{
-        global_state::{CommitResult, StateProvider},
-        protocol_data::ProtocolData,
-    },
+    shared::newtypes::CorrelationId,
+    storage::global_state::StateProvider,
 };
 
+/// Represents a successfuly executed upgrade.
 #[derive(Debug, Clone)]
-pub enum UpgradeResult {
-    RootNotFound,
-    KeyNotFound(Key),
-    TypeMismatch(TypeMismatch),
-    Serialization(bytesrepr::Error),
-    Success {
-        post_state_hash: Blake2bHash,
-        effect: ExecutionEffect,
-    },
+pub struct UpgradeSuccess {
+    /// New state root hash generated after effects were applied.
+    pub post_state_hash: Digest,
+    /// Effects of executing an upgrade request.
+    pub execution_effect: ExecutionEffect,
 }
 
-impl fmt::Display for UpgradeResult {
+impl fmt::Display for UpgradeSuccess {
     fn fmt(&self, f: &mut fmt::Formatter) -> Result<(), fmt::Error> {
-        match self {
-            UpgradeResult::RootNotFound => write!(f, "Root not found"),
-            UpgradeResult::KeyNotFound(key) => write!(f, "Key not found: {}", key),
-            UpgradeResult::TypeMismatch(type_mismatch) => {
-                write!(f, "Type mismatch: {:?}", type_mismatch)
-            }
-            UpgradeResult::Serialization(error) => write!(f, "Serialization error: {:?}", error),
-            UpgradeResult::Success {
-                post_state_hash,
-                effect,
-            } => write!(f, "Success: {} {:?}", post_state_hash, effect),
-        }
-    }
-}
-
-impl UpgradeResult {
-    pub fn from_commit_result(commit_result: CommitResult, effect: ExecutionEffect) -> Self {
-        match commit_result {
-            CommitResult::RootNotFound => UpgradeResult::RootNotFound,
-            CommitResult::KeyNotFound(key) => UpgradeResult::KeyNotFound(key),
-            CommitResult::TypeMismatch(type_mismatch) => UpgradeResult::TypeMismatch(type_mismatch),
-            CommitResult::Serialization(error) => UpgradeResult::Serialization(error),
-            CommitResult::Success { state_root, .. } => UpgradeResult::Success {
-                post_state_hash: state_root,
-                effect,
-            },
-        }
-    }
-
-    pub fn is_success(&self) -> bool {
-        matches!(&self, UpgradeResult::Success { .. })
+        write!(
+            f,
+            "Success: {} {:?}",
+            self.post_state_hash, self.execution_effect
+        )
     }
 }
 
+/// Represents the configuration of a protocol upgrade.
 #[derive(Debug, Clone, PartialEq, Eq)]
 pub struct UpgradeConfig {
-    pre_state_hash: Blake2bHash,
+    pre_state_hash: Digest,
     current_protocol_version: ProtocolVersion,
     new_protocol_version: ProtocolVersion,
-    wasm_config: Option<WasmConfig>,
-    system_config: Option<SystemConfig>,
     activation_point: Option<EraId>,
     new_validator_slots: Option<u32>,
     new_auction_delay: Option<u64>,
@@ -92,13 +55,12 @@ pub struct UpgradeConfig {
 }
 
 impl UpgradeConfig {
+    /// Create new upgrade config.
     #[allow(clippy::too_many_arguments)]
     pub fn new(
-        pre_state_hash: Blake2bHash,
+        pre_state_hash: Digest,
         current_protocol_version: ProtocolVersion,
         new_protocol_version: ProtocolVersion,
-        wasm_config: Option<WasmConfig>,
-        system_config: Option<SystemConfig>,
         activation_point: Option<EraId>,
         new_validator_slots: Option<u32>,
         new_auction_delay: Option<u64>,
@@ -111,8 +73,6 @@ impl UpgradeConfig {
             pre_state_hash,
             current_protocol_version,
             new_protocol_version,
-            wasm_config,
-            system_config,
             activation_point,
             new_validator_slots,
             new_auction_delay,
@@ -123,79 +83,97 @@ impl UpgradeConfig {
         }
     }
 
-    pub fn pre_state_hash(&self) -> Blake2bHash {
+    /// Returns the current state root state hash
+    pub fn pre_state_hash(&self) -> Digest {
         self.pre_state_hash
     }
 
+    /// Returns current protocol version of this upgrade.
     pub fn current_protocol_version(&self) -> ProtocolVersion {
         self.current_protocol_version
     }
 
+    /// Returns new protocol version of this upgrade.
     pub fn new_protocol_version(&self) -> ProtocolVersion {
         self.new_protocol_version
     }
 
-    pub fn wasm_config(&self) -> Option<&WasmConfig> {
-        self.wasm_config.as_ref()
-    }
-
-    pub fn system_config(&self) -> Option<&SystemConfig> {
-        self.system_config.as_ref()
-    }
-
+    /// Returns activation point in eras.
     pub fn activation_point(&self) -> Option<EraId> {
         self.activation_point
     }
 
+    /// Returns new validator slots if specified.
     pub fn new_validator_slots(&self) -> Option<u32> {
         self.new_validator_slots
     }
 
+    /// Returns new auction delay if specified.
     pub fn new_auction_delay(&self) -> Option<u64> {
         self.new_auction_delay
     }
 
+    /// Returns new locked funds period if specified.
     pub fn new_locked_funds_period_millis(&self) -> Option<u64> {
         self.new_locked_funds_period_millis
     }
 
+    /// Returns new round seigniorage rate if specified.
     pub fn new_round_seigniorage_rate(&self) -> Option<Ratio<u64>> {
         self.new_round_seigniorage_rate
     }
 
+    /// Returns new unbonding delay if specified.
     pub fn new_unbonding_delay(&self) -> Option<u64> {
         self.new_unbonding_delay
     }
 
+    /// Returns new map of emergency global state updates.
     pub fn global_state_update(&self) -> &BTreeMap<Key, StoredValue> {
         &self.global_state_update
     }
 
-    pub fn with_pre_state_hash(&mut self, pre_state_hash: Blake2bHash) {
+    /// Sets new pre state hash.
+    pub fn with_pre_state_hash(&mut self, pre_state_hash: Digest) {
         self.pre_state_hash = pre_state_hash;
     }
 }
 
+/// Represents outcomes of a failed protocol upgrade.
 #[derive(Clone, Error, Debug)]
 pub enum ProtocolUpgradeError {
+    /// Error validating a protocol upgrade config.
     #[error("Invalid upgrade config")]
     InvalidUpgradeConfig,
+    /// Unable to retrieve a system contract.
     #[error("Unable to retrieve system contract: {0}")]
     UnableToRetrieveSystemContract(String),
+    /// Unable to retrieve a system contract package.
     #[error("Unable to retrieve system contract package: {0}")]
     UnableToRetrieveSystemContractPackage(String),
+    /// Unable to disable previous version of a system contract.
     #[error("Failed to disable previous version of system contract: {0}")]
     FailedToDisablePreviousVersion(String),
-    #[error(transparent)]
-    Bytesrepr(#[from] bytesrepr::Error),
+    /// (De)serialization error.
+    #[error("{0}")]
+    Bytesrepr(bytesrepr::Error),
+    /// Failed to create system contract registry.
+    #[error("Failed to insert system contract registry")]
+    FailedToCreateSystemRegistry,
 }
 
+impl From<bytesrepr::Error> for ProtocolUpgradeError {
+    fn from(error: bytesrepr::Error) -> Self {
+        ProtocolUpgradeError::Bytesrepr(error)
+    }
+}
+
+/// The system upgrader deals with conducting an actual protocol upgrade.
 pub(crate) struct SystemUpgrader<S>
 where
     S: StateProvider,
 {
     new_protocol_version: ProtocolVersion,
-    protocol_data: ProtocolData,
     tracking_copy: Rc<RefCell<TrackingCopy<<S as StateProvider>::Reader>>>,
 }
 
@@ -203,14 +181,13 @@ impl<S> SystemUpgrader<S>
 where
     S: StateProvider,
 {
+    /// Creates new system upgrader instance.
     pub(crate) fn new(
         new_protocol_version: ProtocolVersion,
-        protocol_data: ProtocolData,
         tracking_copy: Rc<RefCell<TrackingCopy<<S as StateProvider>::Reader>>>,
     ) -> Self {
         SystemUpgrader {
             new_protocol_version,
-            protocol_data,
             tracking_copy,
         }
     }
@@ -219,28 +196,27 @@ where
     pub(crate) fn upgrade_system_contracts_major_version(
         &self,
         correlation_id: CorrelationId,
+        mint_hash: &ContractHash,
+        auction_hash: &ContractHash,
+        handle_payment_hash: &ContractHash,
+        standard_payment_hash: &ContractHash,
     ) -> Result<(), ProtocolUpgradeError> {
+        self.store_contract(correlation_id, *mint_hash, MINT, mint::mint_entry_points())?;
         self.store_contract(
             correlation_id,
-            self.protocol_data.mint(),
-            MINT,
-            mint::mint_entry_points(),
-        )?;
-        self.store_contract(
-            correlation_id,
-            self.protocol_data.auction(),
+            *auction_hash,
             AUCTION,
             auction::auction_entry_points(),
         )?;
         self.store_contract(
             correlation_id,
-            self.protocol_data.handle_payment(),
+            *handle_payment_hash,
             HANDLE_PAYMENT,
             handle_payment::handle_payment_entry_points(),
         )?;
         self.store_contract(
             correlation_id,
-            self.protocol_data.standard_payment(),
+            *standard_payment_hash,
             STANDARD_PAYMENT,
             standard_payment::standard_payment_entry_points(),
         )?;
@@ -248,6 +224,7 @@ where
         Ok(())
     }
 
+    /// Store new system contract.
     fn store_contract(
         &self,
         correlation_id: CorrelationId,
