@@ -5,12 +5,12 @@ use tracing::warn;
 use wasmi::ModuleRef;
 
 use casper_types::{
-    account::AccountHash,
+    account::{Account, AccountHash},
     bytesrepr::FromBytes,
     contracts::NamedKeys,
-    system::{auction, handle_payment, mint, CallStackElement},
-    BlockTime, CLTyped, CLValue, ContractPackage, DeployHash, EntryPoint, EntryPointType, Key,
-    Phase, ProtocolVersion, RuntimeArgs,
+    system::{auction, handle_payment, mint, CallStackElement, AUCTION, HANDLE_PAYMENT, MINT},
+    BlockTime, CLTyped, CLValue, ContractPackage, DeployHash, EntryPoint, EntryPointType, Gas, Key,
+    Phase, ProtocolVersion, RuntimeArgs, StoredValue,
 };
 
 use crate::{
@@ -22,10 +22,10 @@ use crate::{
         execution::{address_generator::AddressGenerator, Error},
         runtime::{extract_access_rights_from_keys, instance_and_memory, Runtime},
         runtime_context::{self, RuntimeContext},
-        tracking_copy::TrackingCopy,
+        tracking_copy::{TrackingCopy, TrackingCopyExt},
     },
-    shared::{account::Account, gas::Gas, newtypes::CorrelationId, stored_value::StoredValue},
-    storage::{global_state::StateReader, protocol_data::ProtocolData},
+    shared::newtypes::CorrelationId,
+    storage::global_state::StateReader,
 };
 
 macro_rules! on_fail_charge {
@@ -47,14 +47,14 @@ macro_rules! on_fail_charge {
                 warn!("Execution failed: {:?}", exec_err);
                 return ExecutionResult::Failure {
                     error: exec_err.into(),
-                    effect: Default::default(),
+                    execution_effect: Default::default(),
                     transfers: $transfers,
                     cost: $cost,
                 };
             }
         }
     };
-    ($fn:expr, $cost:expr, $effect:expr, $transfers:expr) => {
+    ($fn:expr, $cost:expr, $execution_effect:expr, $transfers:expr) => {
         match $fn {
             Ok(res) => res,
             Err(e) => {
@@ -62,7 +62,7 @@ macro_rules! on_fail_charge {
                 warn!("Execution failed: {:?}", exec_err);
                 return ExecutionResult::Failure {
                     error: exec_err.into(),
-                    effect: $effect,
+                    execution_effect: $execution_effect,
                     transfers: $transfers,
                     cost: $cost,
                 };
@@ -71,20 +71,28 @@ macro_rules! on_fail_charge {
     };
 }
 
+/// Executor object deals with execution of WASM modules.
 pub struct Executor {
     config: EngineConfig,
 }
 
 #[allow(clippy::too_many_arguments)]
 impl Executor {
+    /// Creates new executor object.
     pub fn new(config: EngineConfig) -> Self {
         Executor { config }
     }
 
+    /// Returns config.
     pub fn config(&self) -> EngineConfig {
         self.config
     }
 
+    /// Executes a WASM module.
+    ///
+    /// This method checks if a given contract hash is a system contract, and then short circuits to
+    /// a specific native implementation of it. Otherwise, a supplied WASM module is executed.
+    #[allow(clippy::too_many_arguments)]
     pub fn exec<R>(
         &self,
         module: Module,
@@ -101,7 +109,6 @@ impl Executor {
         correlation_id: CorrelationId,
         tracking_copy: Rc<RefCell<TrackingCopy<R>>>,
         phase: Phase,
-        protocol_data: ProtocolData,
         system_contract_cache: SystemContractCache,
         contract_package: &ContractPackage,
         call_stack: Vec<CallStackElement>,
@@ -117,7 +124,7 @@ impl Executor {
         let (instance, memory) = on_fail_charge!(instance_and_memory(
             module.clone(),
             protocol_version,
-            protocol_data.wasm_config()
+            self.config.wasm_config()
         ));
 
         let access_rights = {
@@ -142,7 +149,7 @@ impl Executor {
 
         // Snapshot of effects before execution, so in case of error
         // only nonce update can be returned.
-        let effects_snapshot = tracking_copy.borrow().effect();
+        let execution_effect = tracking_copy.borrow().effect();
 
         let context = RuntimeContext::new(
             tracking_copy,
@@ -163,7 +170,7 @@ impl Executor {
             protocol_version,
             correlation_id,
             phase,
-            protocol_data,
+            self.config,
             transfers,
         );
 
@@ -200,7 +207,7 @@ impl Executor {
             ) {
                 Ok(_value) => {
                     return ExecutionResult::Success {
-                        effect: runtime.context().effect(),
+                        execution_effect: runtime.context().effect(),
                         transfers: runtime.context().transfers().to_owned(),
                         cost: runtime.context().gas_counter(),
                     };
@@ -208,7 +215,7 @@ impl Executor {
                 Err(error) => {
                     return ExecutionResult::Failure {
                         error: error.into(),
-                        effect: effects_snapshot,
+                        execution_effect,
                         transfers: runtime.context().transfers().to_owned(),
                         cost: runtime.context().gas_counter(),
                     };
@@ -225,7 +232,7 @@ impl Executor {
             ) {
                 Ok(_value) => {
                     return ExecutionResult::Success {
-                        effect: runtime.context().effect(),
+                        execution_effect: runtime.context().effect(),
                         transfers: runtime.context().transfers().to_owned(),
                         cost: runtime.context().gas_counter(),
                     };
@@ -233,7 +240,7 @@ impl Executor {
                 Err(error) => {
                     return ExecutionResult::Failure {
                         error: error.into(),
-                        effect: effects_snapshot,
+                        execution_effect,
                         transfers: runtime.context().transfers().to_owned(),
                         cost: runtime.context().gas_counter(),
                     };
@@ -250,7 +257,7 @@ impl Executor {
             ) {
                 Ok(_value) => {
                     return ExecutionResult::Success {
-                        effect: runtime.context().effect(),
+                        execution_effect: runtime.context().effect(),
                         transfers: runtime.context().transfers().to_owned(),
                         cost: runtime.context().gas_counter(),
                     }
@@ -258,7 +265,7 @@ impl Executor {
                 Err(error) => {
                     return ExecutionResult::Failure {
                         error: error.into(),
-                        effect: effects_snapshot,
+                        execution_effect,
                         transfers: runtime.context().transfers().to_owned(),
                         cost: runtime.context().gas_counter(),
                     }
@@ -268,17 +275,18 @@ impl Executor {
         on_fail_charge!(
             instance.invoke_export(entry_point_name, &[], &mut runtime),
             runtime.context().gas_counter(),
-            effects_snapshot,
+            execution_effect,
             runtime.context().transfers().to_owned()
         );
 
         ExecutionResult::Success {
-            effect: runtime.context().effect(),
+            execution_effect: runtime.context().effect(),
             transfers: runtime.context().transfers().to_owned(),
             cost: runtime.context().gas_counter(),
         }
     }
 
+    /// Executes a standard payment code natively.
     pub fn exec_standard_payment<R>(
         &self,
         system_module: Module,
@@ -294,7 +302,6 @@ impl Executor {
         correlation_id: CorrelationId,
         tracking_copy: Rc<RefCell<TrackingCopy<R>>>,
         phase: Phase,
-        protocol_data: ProtocolData,
         system_contract_cache: SystemContractCache,
         call_stack: Vec<CallStackElement>,
     ) -> ExecutionResult
@@ -335,7 +342,6 @@ impl Executor {
             correlation_id,
             Rc::clone(&tracking_copy),
             phase,
-            protocol_data,
             system_contract_cache,
             call_stack,
         ) {
@@ -343,30 +349,36 @@ impl Executor {
             Err(error) => {
                 return ExecutionResult::Failure {
                     error: error.into(),
-                    effect: Default::default(),
+                    execution_effect: Default::default(),
                     transfers: Vec::default(),
                     cost: Gas::default(),
                 };
             }
         };
 
-        let effects_snapshot = tracking_copy.borrow().effect();
+        let execution_effect = tracking_copy.borrow().effect();
 
         match runtime.call_host_standard_payment() {
             Ok(()) => ExecutionResult::Success {
-                effect: runtime.context().effect(),
+                execution_effect: runtime.context().effect(),
                 transfers: runtime.context().transfers().to_owned(),
                 cost: runtime.context().gas_counter(),
             },
             Err(error) => ExecutionResult::Failure {
                 error: error.into(),
-                effect: effects_snapshot,
+                execution_effect,
                 transfers: runtime.context().transfers().to_owned(),
                 cost: runtime.context().gas_counter(),
             },
         }
     }
 
+    /// Executes a system contract.
+    ///
+    /// System contracts are implemented as a native code, and no WASM execution is involved at all.
+    /// This approach has a benefit of speed as compared to executing WASM modules.
+    ///
+    /// Returns an optional return value from the system contract call, and an [`ExecutionResult`].
     pub fn exec_system_contract<R, T>(
         &self,
         direct_system_contract_call: DirectSystemContractCall,
@@ -384,7 +396,6 @@ impl Executor {
         correlation_id: CorrelationId,
         tracking_copy: Rc<RefCell<TrackingCopy<R>>>,
         phase: Phase,
-        protocol_data: ProtocolData,
         system_contract_cache: SystemContractCache,
         call_stack: Vec<CallStackElement>,
     ) -> (Option<T>, ExecutionResult)
@@ -393,11 +404,23 @@ impl Executor {
         R::Error: Into<Error>,
         T: FromBytes + CLTyped,
     {
+        // TODO See if these panics can be removed.
+        let system_contract_registry = tracking_copy
+            .borrow_mut()
+            .get_system_contracts(correlation_id)
+            .unwrap_or_else(|error| panic!("Could not retrieve system contracts: {:?}", error));
+
         match direct_system_contract_call {
             DirectSystemContractCall::Slash
             | DirectSystemContractCall::RunAuction
             | DirectSystemContractCall::DistributeRewards => {
-                if Some(protocol_data.auction().value()) != base_key.into_hash() {
+                // TODO See if these panics can be removed.
+                let auction_hash = system_contract_registry
+                    .get(AUCTION)
+                    .expect("should have auction hash")
+                    .to_owned();
+
+                if Some(auction_hash.value()) != base_key.into_hash() {
                     panic!(
                         "{} should only be called with the auction contract",
                         direct_system_contract_call.entry_point_name()
@@ -406,7 +429,11 @@ impl Executor {
             }
             DirectSystemContractCall::FinalizePayment
             | DirectSystemContractCall::GetPaymentPurse => {
-                if Some(protocol_data.handle_payment().value()) != base_key.into_hash() {
+                // TODO See if these panics can be removed.
+                let handle_payment = system_contract_registry
+                    .get(HANDLE_PAYMENT)
+                    .expect("should have handle payment");
+                if Some(handle_payment.value()) != base_key.into_hash() {
                     panic!(
                         "{} should only be called with the handle payment contract",
                         direct_system_contract_call.entry_point_name()
@@ -414,7 +441,11 @@ impl Executor {
                 }
             }
             DirectSystemContractCall::CreatePurse | DirectSystemContractCall::Transfer => {
-                if Some(protocol_data.mint().value()) != base_key.into_hash() {
+                // TODO See if these panics can be removed.
+                let mint_hash = system_contract_registry
+                    .get(MINT)
+                    .expect("should have mint hash");
+                if Some(mint_hash.value()) != base_key.into_hash() {
                     panic!(
                         "{} should only be called with the mint contract",
                         direct_system_contract_call.entry_point_name()
@@ -422,7 +453,12 @@ impl Executor {
                 }
             }
             DirectSystemContractCall::GetEraValidators => {
-                if Some(protocol_data.auction().value()) != base_key.into_hash() {
+                // TODO See if these panics can be removed.
+                let auction_hash = system_contract_registry
+                    .get(AUCTION)
+                    .expect("should have auction hash")
+                    .to_owned();
+                if Some(auction_hash.value()) != base_key.into_hash() {
                     panic!(
                         "{} should only be called with the auction contract",
                         direct_system_contract_call.entry_point_name()
@@ -447,7 +483,7 @@ impl Executor {
 
         // Snapshot of effects before execution, so in case of error only nonce update
         // can be returned.
-        let effect_snapshot = tracking_copy.borrow().effect();
+        let execution_effect = tracking_copy.borrow().effect();
 
         let transfers = Vec::default();
 
@@ -470,14 +506,13 @@ impl Executor {
             correlation_id,
             tracking_copy,
             phase,
-            protocol_data,
             system_contract_cache,
             call_stack,
         ) {
             Ok((instance, runtime)) => (instance, runtime),
             Err(error) => {
                 return ExecutionResult::Failure {
-                    effect: effect_snapshot,
+                    execution_effect,
                     transfers,
                     cost: gas_counter,
                     error: error.into(),
@@ -493,7 +528,7 @@ impl Executor {
             &mut inner_named_keys,
             &runtime_args,
             extra_keys,
-            effect_snapshot,
+            execution_effect,
         );
         *named_keys = inner_named_keys;
         ret
@@ -518,7 +553,6 @@ impl Executor {
         correlation_id: CorrelationId,
         tracking_copy: Rc<RefCell<TrackingCopy<R>>>,
         phase: Phase,
-        protocol_data: ProtocolData,
         system_contract_cache: SystemContractCache,
         call_stack: Vec<CallStackElement>,
     ) -> Result<T, Error>
@@ -549,7 +583,6 @@ impl Executor {
             correlation_id,
             tracking_copy,
             phase,
-            protocol_data,
             system_contract_cache,
             call_stack,
         )?;
@@ -587,7 +620,13 @@ impl Executor {
         Ok(ret)
     }
 
-    pub fn create_runtime<'a, R>(
+    /// Creates new runtime object.
+    ///
+    /// This method also deals with proper initialiation of a WASM module by pre-allocating a memory
+    /// instance, and attaching a host function resolver.
+    ///
+    /// Returns a module and an instance of [`Runtime`] which is ready to execute the WASM modules.
+    pub(crate) fn create_runtime<'a, R>(
         &self,
         module: Module,
         entry_point_type: EntryPointType,
@@ -607,7 +646,6 @@ impl Executor {
         correlation_id: CorrelationId,
         tracking_copy: Rc<RefCell<TrackingCopy<R>>>,
         phase: Phase,
-        protocol_data: ProtocolData,
         system_contract_cache: SystemContractCache,
         call_stack: Vec<CallStackElement>,
     ) -> Result<(ModuleRef, Runtime<'a, R>), Error>
@@ -643,15 +681,12 @@ impl Executor {
             protocol_version,
             correlation_id,
             phase,
-            protocol_data,
+            self.config,
             transfers,
         );
 
-        let (instance, memory) = instance_and_memory(
-            module.clone(),
-            protocol_version,
-            protocol_data.wasm_config(),
-        )?;
+        let (instance, memory) =
+            instance_and_memory(module.clone(), protocol_version, self.config.wasm_config())?;
 
         let runtime = Runtime::new(
             self.config,
@@ -666,14 +701,23 @@ impl Executor {
     }
 }
 
+/// Represents a variant of a system contract call.
 pub enum DirectSystemContractCall {
+    /// Calls auction's `slash` entry point.
     Slash,
+    /// Calls auction's `run_auction` entry point.
     RunAuction,
+    /// Calls auction's `distribute` entry point.
     DistributeRewards,
+    /// Calls handle payment's `finalize` entry point.
     FinalizePayment,
+    /// Calls mint's `create` entry point.
     CreatePurse,
+    /// Calls mint's `transfer` entry point.
     Transfer,
+    /// Calls auction's `get_era_validators` entry point.
     GetEraValidators,
+    /// Calls handle payment's `
     GetPaymentPurse,
 }
 
@@ -759,14 +803,14 @@ impl DirectSystemContractCall {
         match result {
             Ok(value) => match value.into_t() {
                 Ok(ret) => ExecutionResult::Success {
-                    effect: runtime.context().effect(),
+                    execution_effect: runtime.context().effect(),
                     transfers: runtime.context().transfers().to_owned(),
                     cost: runtime.context().gas_counter(),
                 }
                 .take_with_ret(ret),
                 Err(error) => ExecutionResult::Failure {
                     error: Error::CLValue(error).into(),
-                    effect: execution_effect,
+                    execution_effect,
                     transfers: runtime.context().transfers().to_owned(),
                     cost: runtime.context().gas_counter(),
                 }
@@ -774,7 +818,7 @@ impl DirectSystemContractCall {
             },
             Err(error) => ExecutionResult::Failure {
                 error: error.into(),
-                effect: execution_effect,
+                execution_effect,
                 transfers: runtime.context().transfers().to_owned(),
                 cost: runtime.context().gas_counter(),
             }
