@@ -9,16 +9,14 @@ use casper_types::{
             AccountProvider, Auction, Bid, EraInfo, Error, MintProvider, RuntimeProvider,
             StorageProvider, UnbondingPurse,
         },
-        mint, CallStackElement,
+        mint,
     },
-    CLTyped, CLValue, EraId, Key, KeyTag, PublicKey, RuntimeArgs, URef, BLAKE2B_DIGEST_LENGTH,
-    U512,
+    CLTyped, CLValue, EraId, Key, KeyTag, PublicKey, RuntimeArgs, StoredValue, URef,
+    BLAKE2B_DIGEST_LENGTH, U512,
 };
 
 use super::Runtime;
-use crate::{
-    core::execution, shared::stored_value::StoredValue, storage::global_state::StateReader,
-};
+use crate::{core::execution, storage::global_state::StateReader};
 
 impl From<execution::Error> for Option<Error> {
     fn from(exec_error: execution::Error) -> Self {
@@ -119,8 +117,8 @@ where
         self.context.get_caller()
     }
 
-    fn get_immediate_caller(&self) -> Option<&CallStackElement> {
-        Runtime::<'a, R>::get_immediate_caller(self)
+    fn is_allowed_session_caller(&self, account_hash: &AccountHash) -> bool {
+        self.is_allowed_session_caller(account_hash)
     }
 
     fn named_keys_get(&self, name: &str) -> Option<Key> {
@@ -224,19 +222,25 @@ where
     }
 
     fn read_base_round_reward(&mut self) -> Result<U512, Error> {
-        let mint_contract = self.get_mint_contract();
+        let mint_contract = self.get_mint_contract().map_err(|exec_error| {
+            <Option<Error>>::from(exec_error).unwrap_or(Error::MissingValue)
+        })?;
         self.mint_read_base_round_reward(mint_contract)
             .map_err(|exec_error| <Option<Error>>::from(exec_error).unwrap_or(Error::MissingValue))
     }
 
     fn mint(&mut self, amount: U512) -> Result<URef, Error> {
-        let mint_contract = self.get_mint_contract();
+        let mint_contract = self
+            .get_mint_contract()
+            .map_err(|exec_error| <Option<Error>>::from(exec_error).unwrap_or(Error::MintReward))?;
         self.mint_mint(mint_contract, amount)
             .map_err(|exec_error| <Option<Error>>::from(exec_error).unwrap_or(Error::MintReward))
     }
 
     fn reduce_total_supply(&mut self, amount: U512) -> Result<(), Error> {
-        let mint_contract = self.get_mint_contract();
+        let mint_contract = self
+            .get_mint_contract()
+            .map_err(|exec_error| <Option<Error>>::from(exec_error).unwrap_or(Error::MintReward))?;
         self.mint_reduce_total_supply(mint_contract, amount)
             .map_err(|exec_error| <Option<Error>>::from(exec_error).unwrap_or(Error::MintReward))
     }
