@@ -10,34 +10,29 @@ use rand::RngCore;
 
 use casper_types::{
     account::{
-        AccountHash, ActionType, AddKeyFailure, RemoveKeyFailure, SetThresholdFailure, Weight,
+        Account, AccountHash, ActionType, AddKeyFailure, AssociatedKeys, RemoveKeyFailure,
+        SetThresholdFailure, Weight, ACCOUNT_HASH_LENGTH,
     },
     bytesrepr::ToBytes,
     contracts::NamedKeys,
-    AccessRights, BlockTime, CLValue, Contract, DeployHash, EntryPointType, EntryPoints, Key,
-    Phase, ProtocolVersion, RuntimeArgs, URef, KEY_HASH_LENGTH, U512,
+    system::{AUCTION, HANDLE_PAYMENT, MINT, STANDARD_PAYMENT},
+    AccessRights, BlockTime, CLValue, Contract, ContractHash, DeployHash, EntryPointType,
+    EntryPoints, Gas, Key, Phase, ProtocolVersion, PublicKey, RuntimeArgs, SecretKey, StoredValue,
+    URef, KEY_HASH_LENGTH, U256, U512,
 };
 
 use super::{Address, Error, RuntimeContext};
 use crate::{
     core::{
-        execution::AddressGenerator, runtime::extract_access_rights_from_keys,
+        engine_state::{EngineConfig, SystemContractRegistry},
+        execution::AddressGenerator,
+        runtime::extract_access_rights_from_keys,
         tracking_copy::TrackingCopy,
     },
-    shared::{
-        account::{Account, AssociatedKeys},
-        additive_map::AdditiveMap,
-        gas::Gas,
-        newtypes::CorrelationId,
-        stored_value::StoredValue,
-        transform::Transform,
-    },
-    storage::{
-        global_state::{
-            in_memory::{InMemoryGlobalState, InMemoryGlobalStateView},
-            CommitResult, StateProvider,
-        },
-        protocol_data::ProtocolData,
+    shared::{additive_map::AdditiveMap, newtypes::CorrelationId, transform::Transform},
+    storage::global_state::{
+        in_memory::{InMemoryGlobalState, InMemoryGlobalStateView},
+        StateProvider,
     },
 };
 
@@ -45,7 +40,7 @@ const DEPLOY_HASH: [u8; 32] = [1u8; 32];
 const PHASE: Phase = Phase::Session;
 const GAS_LIMIT: u64 = 500_000_000_000_000u64;
 
-static TEST_PROTOCOL_DATA: Lazy<ProtocolData> = Lazy::new(ProtocolData::default);
+static TEST_ENGINE_CONFIG: Lazy<EngineConfig> = Lazy::new(EngineConfig::default);
 
 fn mock_tracking_copy(
     init_key: Key,
@@ -58,15 +53,10 @@ fn mock_tracking_copy(
 
     let mut m = AdditiveMap::new();
     m.insert(init_key, transform);
-    let commit_result = hist
+    let new_hash = hist
         .commit(correlation_id, root_hash, m)
         .expect("Creation of mocked account should be a success.");
 
-    let new_hash = match commit_result {
-        CommitResult::Success { state_root, .. } => state_root,
-        other => panic!("Committing changes to test History failed: {:?}.", other),
-    };
-
     let reader = hist
         .checkout(new_hash)
         .expect("Checkout should not throw errors.")
@@ -135,7 +125,7 @@ fn mock_runtime_context<'a>(
         named_keys,
         access_rights,
         RuntimeArgs::new(),
-        BTreeSet::from_iter(vec![AccountHash::new([0; 32])]),
+        BTreeSet::from_iter(vec![account.account_hash()]),
         account,
         base_key,
         BlockTime::new(0),
@@ -148,7 +138,7 @@ fn mock_runtime_context<'a>(
         ProtocolVersion::V1_0_0,
         CorrelationId::new(),
         Phase::Session,
-        *TEST_PROTOCOL_DATA,
+        *TEST_ENGINE_CONFIG,
         Vec::default(),
     )
 }
@@ -176,8 +166,11 @@ fn test<T, F>(access_rights: HashMap<Address, HashSet<AccessRights>>, query: F)
 where
     F: FnOnce(RuntimeContext<InMemoryGlobalStateView>) -> Result<T, Error>,
 {
+    let secret_key = SecretKey::ed25519_from_bytes([222; SecretKey::ED25519_LENGTH])
+        .expect("should create secret key");
+    let public_key = PublicKey::from(&secret_key);
     let deploy_hash = [1u8; 32];
-    let (base_key, account) = mock_account(AccountHash::new([0u8; 32]));
+    let (base_key, account) = mock_account(public_key.to_account_hash());
 
     let mut named_keys = NamedKeys::new();
     let uref_address_generator = AddressGenerator::new(&deploy_hash, Phase::Session);
@@ -355,6 +348,19 @@ fn contract_key_addable_valid() {
     )));
     tracking_copy.borrow_mut().write(contract_key, contract);
 
+    let default_system_registry = {
+        let mut registry = SystemContractRegistry::new();
+        registry.insert(MINT.to_string(), ContractHash::default());
+        registry.insert(HANDLE_PAYMENT.to_string(), ContractHash::default());
+        registry.insert(STANDARD_PAYMENT.to_string(), ContractHash::default());
+        registry.insert(AUCTION.to_string(), ContractHash::default());
+        StoredValue::CLValue(CLValue::from_t(registry).unwrap())
+    };
+
+    tracking_copy
+        .borrow_mut()
+        .write(Key::SystemContractRegistry, default_system_registry);
+
     let mut named_keys = NamedKeys::new();
     let uref = create_uref(&mut uref_address_generator, AccessRights::WRITE);
     let uref_name = "NewURef".to_owned();
@@ -382,7 +388,7 @@ fn contract_key_addable_valid() {
         ProtocolVersion::V1_0_0,
         CorrelationId::new(),
         PHASE,
-        Default::default(),
+        EngineConfig::default(),
         Vec::default(),
     );
 
@@ -455,7 +461,7 @@ fn contract_key_addable_invalid() {
         ProtocolVersion::V1_0_0,
         CorrelationId::new(),
         PHASE,
-        Default::default(),
+        EngineConfig::default(),
         Vec::default(),
     );
 
@@ -595,7 +601,8 @@ fn manage_associated_keys() {
             _ => panic!("Invalid transform operation found"),
         };
         account
-            .get_associated_key_weight(account_hash)
+            .associated_keys()
+            .get(&account_hash)
             .expect("Account hash wasn't added to associated keys");
 
         let new_weight = Weight::new(100);
@@ -610,7 +617,8 @@ fn manage_associated_keys() {
             _ => panic!("Invalid transform operation found"),
         };
         let value = account
-            .get_associated_key_weight(account_hash)
+            .associated_keys()
+            .get(&account_hash)
             .expect("Account hash wasn't added to associated keys");
 
         assert_eq!(value, &new_weight, "value was not updated");
@@ -628,7 +636,9 @@ fn manage_associated_keys() {
             _ => panic!("Invalid transform operation found"),
         };
 
-        assert!(account.get_associated_key_weight(account_hash).is_none());
+        let actual = account.associated_keys().get(&account_hash);
+
+        assert!(actual.is_none());
 
         // Remove a key that was already removed
         runtime_context
@@ -865,7 +875,7 @@ fn should_meter_for_gas_storage_write() {
     let uref = create_uref(&mut rng, AccessRights::READ_WRITE);
     let access_rights = extract_access_rights_from_keys(vec![uref]);
     let value = StoredValue::CLValue(CLValue::from_t(43_i32).unwrap());
-    let expected_write_cost = TEST_PROTOCOL_DATA
+    let expected_write_cost = TEST_ENGINE_CONFIG
         .wasm_config()
         .storage_costs()
         .calculate_gas_cost(value.serialized_length());
@@ -895,7 +905,7 @@ fn should_meter_for_gas_storage_add() {
     let uref = create_uref(&mut rng, AccessRights::ADD_WRITE);
     let access_rights = extract_access_rights_from_keys(vec![uref]);
     let value = StoredValue::CLValue(CLValue::from_t(43_i32).unwrap());
-    let expected_add_cost = TEST_PROTOCOL_DATA
+    let expected_add_cost = TEST_ENGINE_CONFIG
         .wasm_config()
         .storage_costs()
         .calculate_gas_cost(value.serialized_length());
@@ -919,3 +929,29 @@ fn should_meter_for_gas_storage_add() {
 
     assert_eq!(gas_usage_after, gas_usage_before + expected_add_cost);
 }
+
+#[test]
+fn associated_keys_add_full() {
+    let final_add_result = test(Default::default(), |mut rc| {
+        let associated_keys_before = rc.account().associated_keys().len();
+
+        for count in 0..(rc.engine_config().max_associated_keys() as usize - associated_keys_before)
+        {
+            let account_hash = {
+                let mut addr = [0; ACCOUNT_HASH_LENGTH];
+                U256::from(count).to_big_endian(&mut addr);
+                AccountHash::new(addr)
+            };
+            let weight = Weight::new(count as u8);
+            rc.add_associated_key(account_hash, weight)
+                .unwrap_or_else(|e| panic!("should add key {}: {:?}", count, e));
+        }
+
+        rc.add_associated_key(AccountHash::new([42; 32]), Weight::new(42))
+    });
+
+    assert!(matches!(
+        final_add_result.expect_err("should error out"),
+        Error::AddKeyFailure(AddKeyFailure::MaxKeysLimit)
+    ));
+}
