@@ -13,10 +13,15 @@ use std::{
 use linked_hash_map::LinkedHashMap;
 use thiserror::Error;
 
-use casper_types::{bytesrepr, CLType, CLValue, CLValueError, Key, KeyTag, Tagged, U512};
+use casper_hashing::Digest;
+use casper_types::{
+    bytesrepr, CLType, CLValue, CLValueError, Key, KeyTag, StoredValue, StoredValueTypeMismatch,
+    Tagged, U512,
+};
 
 pub use self::ext::TrackingCopyExt;
 use self::meter::{heap_meter::HeapSize, Meter};
+use super::engine_state::EngineConfig;
 use crate::{
     core::{
         engine_state::{execution_effect::ExecutionEffect, op::Op},
@@ -24,16 +29,12 @@ use crate::{
     },
     shared::{
         additive_map::AdditiveMap,
-        newtypes::{Blake2bHash, CorrelationId},
-        stored_value::StoredValue,
+        newtypes::CorrelationId,
         transform::{self, Transform},
-        TypeMismatch,
     },
     storage::{global_state::StateReader, trie::merkle_proof::TrieMerkleProof},
 };
 
-use super::engine_state::EngineConfig;
-
 #[derive(Debug)]
 pub enum TrackingCopyQueryResult {
     Success {
@@ -220,7 +221,7 @@ pub struct TrackingCopy<R> {
 pub enum AddResult {
     Success,
     KeyNotFound(Key),
-    TypeMismatch(TypeMismatch),
+    TypeMismatch(StoredValueTypeMismatch),
     Serialization(bytesrepr::Error),
 }
 
@@ -231,7 +232,7 @@ impl From<CLValueError> for AddResult {
             CLValueError::Type(type_mismatch) => {
                 let expected = format!("{:?}", type_mismatch.expected);
                 let found = format!("{:?}", type_mismatch.found);
-                AddResult::TypeMismatch(TypeMismatch::new(expected, found))
+                AddResult::TypeMismatch(StoredValueTypeMismatch::new(expected, found))
             }
         }
     }
@@ -349,7 +350,7 @@ impl<R: StateReader<Key, StoredValue>> TrackingCopy<R> {
 
         let type_name = value.type_name();
         let mismatch = || {
-            Ok(AddResult::TypeMismatch(TypeMismatch::new(
+            Ok(AddResult::TypeMismatch(StoredValueTypeMismatch::new(
                 "I32, U64, U128, U256, U512 or (String, Key) tuple".to_string(),
                 type_name,
             )))
@@ -574,32 +575,42 @@ impl<R: StateReader<Key, StoredValue>> StateReader<Key, StoredValue> for &Tracki
     }
 }
 
+/// Error conditions of a proof validation.
 #[derive(Error, Debug, PartialEq, Eq)]
 pub enum ValidationError {
+    /// The path should not have a different length than the proof less one.
     #[error("The path should not have a different length than the proof less one.")]
     PathLengthDifferentThanProofLessOne,
 
+    /// The provided key does not match the key in the proof.
     #[error("The provided key does not match the key in the proof.")]
     UnexpectedKey,
 
+    /// The provided value does not match the value in the proof.
     #[error("The provided value does not match the value in the proof.")]
     UnexpectedValue,
 
+    /// The proof hash is invalid.
     #[error("The proof hash is invalid.")]
     InvalidProofHash,
 
+    /// The path went cold.
     #[error("The path went cold.")]
     PathCold,
 
+    /// (De)serialization error.
     #[error("Serialization error: {0}")]
     BytesRepr(bytesrepr::Error),
 
+    /// Key is not a URef.
     #[error("Key is not a URef")]
     KeyIsNotAURef(Key),
 
+    /// Error converting a stored value to a [`Key`].
     #[error("Failed to convert stored value to key")]
     ValueToCLValueConversion,
 
+    /// CLValue conversion error.
     #[error("{0}")]
     CLValueError(CLValueError),
 }
@@ -616,8 +627,11 @@ impl From<bytesrepr::Error> for ValidationError {
     }
 }
 
+/// Validates proof of the query.
+///
+/// Returns [`ValidationError`] for any of
 pub fn validate_query_proof(
-    hash: &Blake2bHash,
+    hash: &Digest,
     proofs: &[TrieMerkleProof<Key, StoredValue>],
     expected_first_key: &Key,
     path: &[String],
@@ -672,8 +686,9 @@ pub fn validate_query_proof(
     Ok(())
 }
 
+/// Validates a proof of a balance request.
 pub fn validate_balance_proof(
-    hash: &Blake2bHash,
+    hash: &Digest,
     balance_proof: &TrieMerkleProof<Key, StoredValue>,
     expected_purse_key: Key,
     expected_motes: &U512,
