@@ -1,24 +1,15 @@
-#![allow(missing_docs)]
-
+//! The shared logic of the execution engine.
 pub mod additive_map;
 #[macro_use]
-pub mod gas;
-pub mod account;
 pub mod host_function_costs;
 pub mod logging;
-pub mod motes;
 pub mod newtypes;
 pub mod opcode_costs;
-pub mod socket;
 pub mod storage_costs;
-pub mod stored_value;
 pub mod system_config;
 pub mod test_utils;
 pub mod transform;
-mod type_mismatch;
 pub mod utils;
 pub mod wasm;
 pub mod wasm_config;
 pub mod wasm_prep;
-
-pub use type_mismatch::TypeMismatch;
