@@ -1,13 +1,14 @@
-#![allow(missing_docs)]
-
-// modules
+/// Storage errors.
 pub mod error;
+/// Global State.
 pub mod global_state;
-pub mod protocol_data;
-pub mod protocol_data_store;
+/// Store module.
 pub mod store;
+/// Transaction Source.
 pub mod transaction_source;
+/// Merkle Trie implementation.
 pub mod trie;
+/// Merkle Trie storage.
 pub mod trie_store;
 
 const MAX_DBS: u32 = 2;
