@@ -5,18 +5,26 @@ use thiserror::Error;
 
 use casper_types::bytesrepr;
 
-use crate::storage::error::in_memory;
+use crate::storage::{error::in_memory, global_state::CommitError};
 
+/// Error enum representing possible error states in LMDB interactions.
 #[derive(Debug, Clone, Error, PartialEq, Eq)]
 pub enum Error {
+    /// LMDB error returned from underlying `lmdb` crate.
     #[error(transparent)]
     Lmdb(#[from] lmdb_external::Error),
 
+    /// (De)serialization error.
     #[error("{0}")]
     BytesRepr(bytesrepr::Error),
 
+    /// Concurrency error.
     #[error("Another thread panicked while holding a lock")]
     Poison,
+
+    /// Error committing to execution engine.
+    #[error(transparent)]
+    CommitError(#[from] CommitError),
 }
 
 impl wasmi::HostError for Error {}
