@@ -1,4 +1,5 @@
 mod delete;
+mod ee_699;
 mod keys;
 mod proptests;
 mod read;
@@ -6,31 +7,33 @@ mod scan;
 mod synchronize;
 mod write;
 
-use std::{collections::HashMap, convert};
+use std::{collections::HashMap, convert, ops::Not};
 
 use lmdb::DatabaseFlags;
 use tempfile::{tempdir, TempDir};
 
-use crate::shared::newtypes::{Blake2bHash, CorrelationId};
+use casper_hashing::Digest;
 use casper_types::bytesrepr::{self, FromBytes, ToBytes};
 
-use crate::storage::{
-    error::{self, in_memory},
-    transaction_source::{
-        in_memory::InMemoryEnvironment, lmdb::LmdbEnvironment, Readable, Transaction,
-        TransactionSource,
+use crate::{
+    shared::newtypes::CorrelationId,
+    storage::{
+        error::{self, in_memory},
+        transaction_source::{
+            in_memory::InMemoryEnvironment, lmdb::LmdbEnvironment, Readable, Transaction,
+            TransactionSource,
+        },
+        trie::{merkle_proof::TrieMerkleProof, Pointer, Trie},
+        trie_store::{
+            self,
+            in_memory::InMemoryTrieStore,
+            lmdb::LmdbTrieStore,
+            operations::{self, read, read_with_proof, write, ReadResult, WriteResult},
+            TrieStore,
+        },
+        DEFAULT_TEST_MAX_DB_SIZE, DEFAULT_TEST_MAX_READERS,
     },
-    trie::{merkle_proof::TrieMerkleProof, Pointer, Trie},
-    trie_store::{
-        self,
-        in_memory::InMemoryTrieStore,
-        lmdb::LmdbTrieStore,
-        operations::{self, read, read_with_proof, write, ReadResult, WriteResult},
-        TrieStore,
-    },
-    DEFAULT_TEST_MAX_DB_SIZE, DEFAULT_TEST_MAX_READERS,
 };
-use std::ops::Not;
 
 const TEST_KEY_LENGTH: usize = 7;
 
@@ -89,14 +92,14 @@ type HashedTestTrie = HashedTrie<TestKey, TestValue>;
 /// A pairing of a trie element and its hash.
 #[derive(Debug, Clone, PartialEq, Eq)]
 struct HashedTrie<K, V> {
-    hash: Blake2bHash,
+    hash: Digest,
     trie: Trie<K, V>,
 }
 
 impl<K: ToBytes, V: ToBytes> HashedTrie<K, V> {
     pub fn new(trie: Trie<K, V>) -> Result<Self, bytesrepr::Error> {
         let trie_bytes = trie.to_bytes()?;
-        let hash = Blake2bHash::new(&trie_bytes);
+        let hash = Digest::hash(&trie_bytes);
         Ok(HashedTrie { hash, trie })
     }
 }
@@ -215,7 +218,7 @@ const TEST_LEAVES_ADJACENTS: [TestTrie; TEST_LEAVES_LENGTH] = [
     },
 ];
 
-type TrieGenerator<K, V> = fn() -> Result<(Blake2bHash, Vec<HashedTrie<K, V>>), bytesrepr::Error>;
+type TrieGenerator<K, V> = fn() -> Result<(Digest, Vec<HashedTrie<K, V>>), bytesrepr::Error>;
 
 const TEST_TRIE_GENERATORS_LENGTH: usize = 7;
 
@@ -236,10 +239,10 @@ fn hash_test_tries(tries: &[TestTrie]) -> Result<Vec<HashedTestTrie>, bytesrepr:
         .collect()
 }
 
-fn create_0_leaf_trie() -> Result<(Blake2bHash, Vec<HashedTestTrie>), bytesrepr::Error> {
+fn create_0_leaf_trie() -> Result<(Digest, Vec<HashedTestTrie>), bytesrepr::Error> {
     let root = HashedTrie::new(Trie::node(&[]))?;
 
-    let root_hash: Blake2bHash = root.hash;
+    let root_hash: Digest = root.hash;
 
     let parents: Vec<HashedTestTrie> = vec![root];
 
@@ -252,12 +255,12 @@ fn create_0_leaf_trie() -> Result<(Blake2bHash, Vec<HashedTestTrie>), bytesrepr:
     Ok((root_hash, tries))
 }
 
-fn create_1_leaf_trie() -> Result<(Blake2bHash, Vec<HashedTestTrie>), bytesrepr::Error> {
+fn create_1_leaf_trie() -> Result<(Digest, Vec<HashedTestTrie>), bytesrepr::Error> {
     let leaves = hash_test_tries(&TEST_LEAVES[..1])?;
 
     let root = HashedTrie::new(Trie::node(&[(0, Pointer::LeafPointer(leaves[0].hash))]))?;
 
-    let root_hash: Blake2bHash = root.hash;
+    let root_hash: Digest = root.hash;
 
     let parents: Vec<HashedTestTrie> = vec![root];
 
@@ -271,7 +274,7 @@ fn create_1_leaf_trie() -> Result<(Blake2bHash, Vec<HashedTestTrie>), bytesrepr:
     Ok((root_hash, tries))
 }
 
-fn create_2_leaf_trie() -> Result<(Blake2bHash, Vec<HashedTestTrie>), bytesrepr::Error> {
+fn create_2_leaf_trie() -> Result<(Digest, Vec<HashedTestTrie>), bytesrepr::Error> {
     let leaves = hash_test_tries(&TEST_LEAVES[..2])?;
 
     let node = HashedTrie::new(Trie::node(&[
@@ -300,7 +303,7 @@ fn create_2_leaf_trie() -> Result<(Blake2bHash, Vec<HashedTestTrie>), bytesrepr:
     Ok((root_hash, tries))
 }
 
-fn create_3_leaf_trie() -> Result<(Blake2bHash, Vec<HashedTestTrie>), bytesrepr::Error> {
+fn create_3_leaf_trie() -> Result<(Digest, Vec<HashedTestTrie>), bytesrepr::Error> {
     let leaves = hash_test_tries(&TEST_LEAVES[..3])?;
 
     let node_1 = HashedTrie::new(Trie::node(&[
@@ -339,7 +342,7 @@ fn create_3_leaf_trie() -> Result<(Blake2bHash, Vec<HashedTestTrie>), bytesrepr:
     Ok((root_hash, tries))
 }
 
-fn create_4_leaf_trie() -> Result<(Blake2bHash, Vec<HashedTestTrie>), bytesrepr::Error> {
+fn create_4_leaf_trie() -> Result<(Digest, Vec<HashedTestTrie>), bytesrepr::Error> {
     let leaves = hash_test_tries(&TEST_LEAVES[..4])?;
 
     let node_1 = HashedTrie::new(Trie::node(&[
@@ -383,7 +386,7 @@ fn create_4_leaf_trie() -> Result<(Blake2bHash, Vec<HashedTestTrie>), bytesrepr:
     Ok((root_hash, tries))
 }
 
-fn create_5_leaf_trie() -> Result<(Blake2bHash, Vec<HashedTestTrie>), bytesrepr::Error> {
+fn create_5_leaf_trie() -> Result<(Digest, Vec<HashedTestTrie>), bytesrepr::Error> {
     let leaves = hash_test_tries(&TEST_LEAVES[..5])?;
 
     let node_1 = HashedTrie::new(Trie::node(&[
@@ -432,7 +435,7 @@ fn create_5_leaf_trie() -> Result<(Blake2bHash, Vec<HashedTestTrie>), bytesrepr:
     Ok((root_hash, tries))
 }
 
-fn create_6_leaf_trie() -> Result<(Blake2bHash, Vec<HashedTestTrie>), bytesrepr::Error> {
+fn create_6_leaf_trie() -> Result<(Digest, Vec<HashedTestTrie>), bytesrepr::Error> {
     let leaves = hash_test_tries(&TEST_LEAVES)?;
 
     let node_1 = HashedTrie::new(Trie::node(&[
@@ -481,7 +484,7 @@ fn create_6_leaf_trie() -> Result<(Blake2bHash, Vec<HashedTestTrie>), bytesrepr:
     Ok((root_hash, tries))
 }
 
-fn create_6_leaf_corrupt_trie() -> Result<(Blake2bHash, Vec<HashedTestTrie>), bytesrepr::Error> {
+fn create_6_leaf_corrupt_trie() -> Result<(Digest, Vec<HashedTestTrie>), bytesrepr::Error> {
     let leaves = hash_test_tries(&TEST_LEAVES)?;
 
     let node_1 = HashedTrie::new(Trie::node(&[
@@ -519,7 +522,7 @@ fn create_6_leaf_corrupt_trie() -> Result<(Blake2bHash, Vec<HashedTestTrie>), by
     let root_hash = root.hash;
 
     let mut corrupt_node_3: HashedTestTrie = HashedTrie::new(Trie::node(&[
-        (0, Pointer::NodePointer(Blake2bHash::new(b"yep"))),
+        (0, Pointer::NodePointer(Digest::hash(b"yep"))),
         (2, Pointer::LeafPointer(leaves[2].hash)),
     ]))?;
 
@@ -580,6 +583,7 @@ impl LmdbTestContext {
             &_temp_dir.path().to_path_buf(),
             DEFAULT_TEST_MAX_DB_SIZE,
             DEFAULT_TEST_MAX_READERS,
+            true,
         )?;
         let store = LmdbTrieStore::new(&environment, None, DatabaseFlags::empty())?;
         put_tries::<_, _, _, _, error::Error>(&environment, &store, tries)?;
@@ -632,7 +636,7 @@ fn check_leaves_exist<K, V, T, S, E>(
     correlation_id: CorrelationId,
     txn: &T,
     store: &S,
-    root: &Blake2bHash,
+    root: &Digest,
     leaves: &[Trie<K, V>],
 ) -> Result<Vec<bool>, E>
 where
@@ -662,7 +666,7 @@ fn check_merkle_proofs<K, V, T, S, E>(
     correlation_id: CorrelationId,
     txn: &T,
     store: &S,
-    root: &Blake2bHash,
+    root: &Digest,
     leaves: &[Trie<K, V>],
 ) -> Result<Vec<bool>, E>
 where
@@ -700,7 +704,7 @@ fn check_keys<K, V, T, S, E>(
     correlation_id: CorrelationId,
     txn: &T,
     store: &S,
-    root: &Blake2bHash,
+    root: &Digest,
     leaves: &[Trie<K, V>],
 ) -> bool
 where
@@ -734,7 +738,7 @@ fn check_leaves<'a, K, V, R, S, E>(
     correlation_id: CorrelationId,
     environment: &'a R,
     store: &S,
-    root: &Blake2bHash,
+    root: &Digest,
     present: &[Trie<K, V>],
     absent: &[Trie<K, V>],
 ) -> Result<(), E>
@@ -788,7 +792,7 @@ fn write_leaves<'a, K, V, R, S, E>(
     correlation_id: CorrelationId,
     environment: &'a R,
     store: &S,
-    root_hash: &Blake2bHash,
+    root_hash: &Digest,
     leaves: &[Trie<K, V>],
 ) -> Result<Vec<WriteResult>, E>
 where
@@ -830,7 +834,7 @@ fn check_pairs_proofs<'a, K, V, R, S, E>(
     correlation_id: CorrelationId,
     environment: &'a R,
     store: &S,
-    root_hashes: &[Blake2bHash],
+    root_hashes: &[Digest],
     pairs: &[(K, V)],
 ) -> Result<bool, E>
 where
@@ -865,7 +869,7 @@ fn check_pairs<'a, K, V, R, S, E>(
     correlation_id: CorrelationId,
     environment: &'a R,
     store: &S,
-    root_hashes: &[Blake2bHash],
+    root_hashes: &[Digest],
     pairs: &[(K, V)],
 ) -> Result<bool, E>
 where
@@ -911,9 +915,9 @@ fn write_pairs<'a, K, V, R, S, E>(
     correlation_id: CorrelationId,
     environment: &'a R,
     store: &S,
-    root_hash: &Blake2bHash,
+    root_hash: &Digest,
     pairs: &[(K, V)],
-) -> Result<Vec<Blake2bHash>, E>
+) -> Result<Vec<Digest>, E>
 where
     K: ToBytes + FromBytes + Clone + Eq + std::fmt::Debug,
     V: ToBytes + FromBytes + Clone + Eq,
@@ -947,9 +951,9 @@ fn writes_to_n_leaf_empty_trie_had_expected_results<'a, K, V, R, S, E>(
     correlation_id: CorrelationId,
     environment: &'a R,
     store: &S,
-    states: &[Blake2bHash],
+    states: &[Digest],
     test_leaves: &[Trie<K, V>],
-) -> Result<Vec<Blake2bHash>, E>
+) -> Result<Vec<Digest>, E>
 where
     K: ToBytes + FromBytes + Clone + Eq + std::fmt::Debug + Copy + Ord,
     V: ToBytes + FromBytes + Clone + Eq + std::fmt::Debug + Copy,
@@ -973,7 +977,7 @@ where
         WriteResult::Written(root_hash) => root_hash,
         _ => panic!("write_leaves resulted in non-write"),
     })
-    .collect::<Vec<Blake2bHash>>();
+    .collect::<Vec<Digest>>();
 
     states.extend(hashes);
 
@@ -991,7 +995,7 @@ impl InMemoryEnvironment {
     pub fn dump<K, V>(
         &self,
         maybe_name: Option<&str>,
-    ) -> Result<HashMap<Blake2bHash, Trie<K, V>>, in_memory::Error>
+    ) -> Result<HashMap<Digest, Trie<K, V>>, in_memory::Error>
     where
         K: FromBytes,
         V: FromBytes,
@@ -1002,11 +1006,11 @@ impl InMemoryEnvironment {
         let data = self.data(Some(&name))?.unwrap();
         data.into_iter()
             .map(|(hash_bytes, trie_bytes)| {
-                let hash: Blake2bHash = bytesrepr::deserialize(hash_bytes.to_vec())?;
+                let hash: Digest = bytesrepr::deserialize(hash_bytes.to_vec())?;
                 let trie: Trie<K, V> = bytesrepr::deserialize(trie_bytes.to_vec())?;
                 Ok((hash, trie))
             })
-            .collect::<Result<HashMap<Blake2bHash, Trie<K, V>>, bytesrepr::Error>>()
+            .collect::<Result<HashMap<Digest, Trie<K, V>>, bytesrepr::Error>>()
             .map_err(Into::into)
     }
 }
